@@ -46,7 +46,7 @@ static gboolean gpa_option_menu_mnemonic_activate (GtkWidget *widget,
 						   gboolean group_cycling);
 static GPAWidgetClass *parent_class;
 
-GtkType
+GType
 gpa_option_menu_get_type (void)
 {
 	static GType type = 0;
