@@ -205,7 +205,7 @@ static gboolean
 node_to_iter (GtkTreeModel *model, GPANode *node, GtkTreeIter *iter)
 {
 	if (!gtk_tree_model_get_iter_first (model, iter))
-	  return FALSE;
+		return FALSE;
 	do {
 		GPANode *tem;
 		gtk_tree_model_get (model, iter, 0, &tem, -1);
@@ -444,11 +444,15 @@ gpa_printer_selector_printer_added_cb (GPANode *parent, GPANode *child,
 {
 	GtkTreeIter iter;
 
+	GDK_THREADS_ENTER ();
+
 	g_return_if_fail (node_to_iter (ps->model, child, &iter) == FALSE);
 
 	gtk_list_store_append (GTK_LIST_STORE (ps->model), &iter);
 	gpa_printer_selector_sync_printer (GTK_LIST_STORE (ps->model), &iter,
 					   GPA_PRINTER (child));
+
+	GDK_THREADS_LEAVE ();
 }
 
 static void
@@ -456,10 +460,14 @@ gpa_printer_selector_printer_removed_cb (GPANode *parent, GPANode *child,
 					 GPAPrinterSelector *ps)
 {
 	GtkTreeIter iter;
+
+	GDK_THREADS_ENTER ();
 	
 	g_return_if_fail (node_to_iter (ps->model, child, &iter));
 	
 	gtk_list_store_remove (GTK_LIST_STORE (ps->model), &iter);
+
+	GDK_THREADS_LEAVE ();
 }
 
 static gboolean
@@ -495,11 +503,13 @@ gpa_printer_selector_construct (GPAWidget *gpa)
 	if (ps->config != NULL)
 		default_printer = GPA_REFERENCE_REFERENCE (GPA_CONFIG (ps->config)->printer);
 
-	g_return_val_if_fail (node_to_iter (ps->model, default_printer, &iter), FALSE);
-	gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (ps->sortmodel),
-							&sort_iter, &iter);
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (ps->treeview));
-	gtk_tree_selection_select_iter (selection, &sort_iter);
+	if (default_printer != NULL &&
+	    node_to_iter (ps->model, default_printer, &iter), FALSE) {
+		gtk_tree_model_sort_convert_child_iter_to_iter (
+			GTK_TREE_MODEL_SORT (ps->sortmodel), &sort_iter, &iter);
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (ps->treeview));
+		gtk_tree_selection_select_iter (selection, &sort_iter);
+	}
 
 	return TRUE;
 }
