@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  gpa-transport-selector.c: A print transport selector
  *
@@ -93,7 +93,7 @@ gpa_transport_selector_init (GPATransportSelector *ts)
 
 	hbox = (GtkBox *) gtk_hbox_new (FALSE, 4);
 
-	ts->menu = gtk_option_menu_new ();
+	ts->combo = gtk_combo_box_new ();
 	ts->file_name   = g_strdup ("");
 	ts->file_name_force = FALSE;
 	ts->file_name_label   = gtk_label_new ("");
@@ -105,7 +105,7 @@ gpa_transport_selector_init (GPATransportSelector *ts)
 	g_signal_connect (G_OBJECT (ts->custom_entry), "changed", (GCallback)
 			  gpa_transport_selector_custom_entry_changed_cb, ts);
 
-	gtk_box_pack_start (hbox, ts->menu,         FALSE, FALSE, 0);
+	gtk_box_pack_start (hbox, GTK_WIDGET (ts->combo),	FALSE, FALSE, 0);
 	gtk_box_pack_start (hbox, ts->file_button,   FALSE, FALSE, 0);
 	gtk_box_pack_start (hbox, ts->file_name_label,  FALSE, FALSE, 0);
 	gtk_box_pack_start (hbox, ts->custom_entry, FALSE, FALSE, 0);
@@ -358,11 +358,15 @@ gpa_transport_selector_update_widgets (GPATransportSelector *ts)
 }
 
 static void
-gpa_transport_selector_item_activate_cb (GtkWidget *item, GPATransportSelector *ts)
+gpa_transport_selector_item_activate_cb (GPATransportSelector *ts)
 {
+	GtkTreeIter   iter;
 	GPANode *node;
 
-	node =  g_object_get_data (G_OBJECT (item), "node");
+	gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ts->combo), &iter);
+	gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (ts->combo)),
+		&iter, 1, &node, -1);
+
 	ts->updating = TRUE;
 	gpa_node_set_value (ts->node, gpa_node_id (node));
 	ts->updating = FALSE;
@@ -371,67 +375,55 @@ gpa_transport_selector_item_activate_cb (GtkWidget *item, GPATransportSelector *
 }
 
 static void
-gpa_transport_selector_rebuild_menu (GPATransportSelector *ts)
+gpa_transport_selector_rebuild_combo (GPATransportSelector *ts)
 {
-	GtkWidget *menu, *item;
-	GPANode *option, *child;
-	guchar *def = NULL;
+	GtkListStore	*model;
+	GtkTreeIter	 iter;
+	GPANode *child, *next, *option = NULL;
+	guchar *name, *def = NULL;
 	gint pos = 0;
 	gint sel = -1;
 
-	menu = gtk_menu_new ();
-	gtk_widget_show (menu);
-
+	model = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_OBJECT);
 	if (ts->node != NULL) {
 		option = GPA_KEY (ts->node)->option;
 		def = gpa_node_get_value (ts->node);
 		
-		child = gpa_node_get_child (option, NULL);
-		while (child) {
-			const guchar *id;
-			guchar *name;
-			
+		for (child = gpa_node_get_child (option, NULL); child != NULL ; child = next) {
+			gtk_list_store_append (model, &iter);
 			name = gpa_node_get_value (child);
-			item = gtk_menu_item_new_with_label (name);
+			gtk_list_store_set (model, &iter,
+					    0, name,
+					    1, child,
+					    -1);
 			g_free (name);
 			
-			id = gpa_node_id (child);
-			
-			g_signal_connect 
-				(G_OBJECT (item), "activate", 
-				 (GCallback) 
-				 gpa_transport_selector_item_activate_cb, ts);
-			g_object_set_data_full 
-				(G_OBJECT (item), "node", child, 
-				 (GtkDestroyNotify) gpa_node_unref);
-			gtk_widget_show (item);
-			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 			if (GPA_NODE_ID_COMPARE (child, def))
 				sel = pos;
 			
 			pos++;
-			child = gpa_node_get_child (option, child);
+			next = gpa_node_get_child (option, child);
+			gpa_node_unref (child);
 		}
+		if (def)
+			g_free (def);
+	}
+
+	if (pos <= 1)
+		gtk_widget_hide (ts->combo);
+	else
+		gtk_widget_show (ts->combo);
 		if (sel == -1) {
 			g_warning ("rebuild_menu_cb, could not set value "
 				   "of %s to %s",
 				   gpa_node_id (option), def);
 			sel = 0;
 		}
-	}
-	if (pos < 1) {
-		item = gtk_menu_item_new_with_label 
-			(_("No options are defined"));
-		gtk_widget_set_sensitive (item, FALSE);
-		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	}
-
-	gtk_widget_show (menu);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (ts->menu), menu);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (ts->menu), sel);
-	if (def)
-		g_free (def);
+	ts->updating = TRUE;
+	gtk_combo_box_set_model (GTK_COMBO_BOX (ts->combo), GTK_TREE_MODEL (model));
+	if (pos > 0)
+		gtk_combo_box_set_active (GTK_COMBO_BOX (ts->combo), sel);
+	ts->updating = FALSE;
 
 	gpa_transport_selector_update_widgets (ts);
 }
@@ -452,7 +444,7 @@ gpa_transport_selector_config_modified_cb (GPANode *node, guint flags, GPATransp
 {
 	gpa_transport_selector_disconnect (ts);
 	gpa_transport_selector_connect (ts);
-	gpa_transport_selector_rebuild_menu (ts);
+	gpa_transport_selector_rebuild_combo (ts);
 }
 
 static void
@@ -461,13 +453,14 @@ gpa_transport_selector_node_modified_cb (GPANode *node, guint flags, GPATranspor
 	if (ts->updating)
 		return;
 	
-	gpa_transport_selector_rebuild_menu (ts);
+	gpa_transport_selector_rebuild_combo (ts);
 }
 
 static gint
 gpa_transport_selector_construct (GPAWidget *gpaw)
 {
 	GPATransportSelector *ts;
+        GtkCellRenderer *renderer;
 
 	ts = GPA_TRANSPORT_SELECTOR (gpaw);
 	ts->config  = GNOME_PRINT_CONFIG_NODE (gpaw->config);
@@ -475,7 +468,16 @@ gpa_transport_selector_construct (GPAWidget *gpaw)
 					       gpa_transport_selector_config_modified_cb, ts);
 
 	gpa_transport_selector_connect (ts);
-	gpa_transport_selector_rebuild_menu (ts);
+	gpa_transport_selector_rebuild_combo (ts);
+
+	g_signal_connect_swapped (G_OBJECT (ts->combo),
+		"changed", 
+		G_CALLBACK (gpa_transport_selector_item_activate_cb), ts);
+        renderer = gtk_cell_renderer_text_new ();
+        gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ts->combo),
+		renderer, TRUE);
+        gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (ts->combo),
+		renderer, "text", 0, NULL);
 
 	return TRUE;
 }
